<?php

/**
 * Classe de gestion d'un glossaire d'acronymes, abbréviations et définitions de mots
 * Remerciements tout particuliers à DayPass et Olivier de PHPScripts-fr.net
 * qui m'ont apporté une aide précieuse pour réaliser cette petite application
 *
 * @author Hugo Hamon <webmaster@apprendre-php>
 * @version 0.0.1
 */
class Glossaire 
{
	/**
	 * Tableau des tags html autorisés
	 *
	 * @var array $htmlTags tableau des tags html autorisés
	 * @access protected
	 */
	protected $htmlTags;

	/**
	 * Tableau des tags html transformés
	 *
	 * @var array $tags tableau des tags html transformés
	 * @access protected
	 */
	protected $tags;

	/**
	 * Tableau des mots du glossaire
	 *
	 * @var array $words tableau des mots du glossaire
	 * @access protected
	 */
	protected $words;

	/**
	 * Classe CSS à appliquer sur le mot
	 *
	 * @var string $cssClass classe CSS à appliquer sur le mot
	 * @access protected
	 */
	protected $cssClass;

	/**
	 * Constructeur de la classe sans paramètre
	 *
	 * @param void
	 * @return void
	 * @access public
	 */
	public function __construct() 
	{
		$this->htmlTags = array('acronym','abbr','dfn');
		$this->words = array();
		$this->tags = array();
		$this->cssClass = '';
	}
	
	/**
	 * Destructeur de l'objet
	 *
	 * @param void
	 * @return void
	 * @access public
	 */
	public function __destruct() 
	{
		
	}
	
	/**
	 * Définit tout le tableau de mots
	 *
	 * @param array $words le tableau de mots du glossaire
	 * @return void
	 * @access public
	 */
	public function setAllWords(&$words) 
	{
		$this->words = $words;
	}

	/**
	 * Définit la classe CSS à appliquer sur le mot
	 *
	 * @param string $cssClass classe CSS à appliquer
	 * @return void
	 * @access public
	 */
	public function setCssClass($cssClass)
	{
		$this->cssClass = $cssClass;
	}

	/**
	 * Ajoute un mot au glossaire
	 *
	 * @param string $word le mot à trouver
	 * @param string $definition la définition du mot
	 * @param string $tag son tag html associé (acronym, abbr, dfn)
	 * @return void
	 * @access public
	 */
	public function setWord($word, $definition, $tag) 
	{
		if(!isset($this->words[$word]) && in_array(strtolower($tag), $this->htmlTags)) {
			$this->words[$word][0] = $definition;
			$this->words[$word][1] = strtolower($tag);
		}
	}
	
	/**
	 * Retourne le tableau des tags HTML autorisés
	 *
	 * @param void
	 * @return array $htmlTags le tableau des tags html autorisés
	 * @access public
	 */
	public function getHtmlTags()
	{
	
		return $this->htmlTags;
	}

	/**
	 * Retourne le tableau de mots du glossaire
	 *
	 * @param void
	 * @return array $words le tableau de mots
	 * @access public
	 */
	public function getWords()
	{
		return $this->words;
	}

	/**
	 * Retourne le tableau d'un mot en particulier
	 *
	 * @param string $word le mot dont on veut avoir les attributs
	 * @return array $attributes le tableau des attributs du mot
	 * @access public
	 */
	public function getWord($word)
	{
		if(isset($this->words[$word])) {
			return $this->words[$word];
		}
		return array();
	}

	/**
	 * Retourne la dernière classe CSS à appliquer
	 *
	 * @param void
	 * @return string $cssClass la classe CSS à appliquer
	 * @access public
	 */
	public function getCssClass()
	{
		return $this->cssClass;
	}

	/**
	 * Parse la chaine, c'est-à-dire remplace les mots du dictionnaire
	 * par leur équivalence en HTML
	 *
	 * @param string $text le texte dans lequel on doit remplacer les mots trouvés
	 * @return string $text le texte correctement formaté
	 * @access public
	 */
	public function parseString($text) 
	{
		if(sizeof($this->words)>0) {
			$text = $this->_replaceWordsByEquivalents($text);
		}
		return $text;
	}

	/**
	 * Définit le motif à matcher dans la chaine
	 *
	 * @param string $word le mot à matcher
	 * @return string $pattern le pattern final à matcher
	 * @access protected
	 */
	protected function _setPattern($word)
	{
		return $pattern = '`\b'.$word.'\b`';
	}

	/**
	 * Définit le motif de remplacement d'un mot
	 *
	 * @param string $word le mot à remplacer
	 * @param array $attributes les attributs du mot
	 * @return string $replacement le pattern de remplacement
	 * @access protected
	 */
	protected function _setReplacement($word, $attributes)
	{
		$replacement = '<'. htmlspecialchars($attributes[1]) .' title="'. htmlspecialchars($attributes[0]) .'"';
		if(!empty($this->cssClass)) $replacement.= ' class="'. $this->cssClass .'"';
		$replacement.='>'. htmlspecialchars($word) .'</'. $attributes[1] .'>';
		return $replacement;
	}
	

	/**
	 * Remplace tous les mots par leur équivalents HTML
	 *
	 * @param string $text le texte dans lequel on doit remplacer les mots trouvés
	 * @return string $text le texte correctement formaté
	 * @access protected
	 */
	protected function _replaceWordsByEquivalents($text)
	{
		foreach($this->words as $word => $attributes)
		{
			$text = $this->_replaceWordByEquivalent($word, $attributes, $text);
		}
		return $text;
	}

	/**
	 * Remplace un mot par son  équivalents HTML dans une chaine contenant du code HTML
	 *
	 * @param string $word le mot à remplacer
	 * @param array $attributes le tableau des attributs du mot
	 * @param string $text le texte dans lequel on doit remplacer les mots trouvés
	 * @return string $text le texte correctement formaté
	 * @access protected
	 */
	protected function _replaceWordByEquivalent($word, $attributes, $text) 
	{
		// Préparation du motif à matcher et du remplacement
		$pattern = $this->_setPattern($word);
		$replacement = $this->_setReplacement($word, $attributes);

		// Remplacement du mot
		$text = preg_replace_callback('`<[^>]+>`', array($this, '_saveTags'), $text); 
		$text = preg_replace($pattern, $replacement, $text);
		$text = preg_replace_callback('`<([0-9]+)>`', array($this, '_restoreTags'), $text); 
		return $text;
	}

	/**
	 * Protège les tags HTML déjà présents pour éviter de remplacer des mot
	 * du dictionnaire qui pourrait se trouver dans le tag HTML
	 *
	 * @param array $matches tableau des tags remplacés par preg_replace_callback()
	 * @return string $text le texte avec les tags html protégés
	 * @access protected
	 */
	protected function _saveTags($matches) 
	{
		$i = count($this->tags);
        $this->tags[$i] = $matches[0];
        return '<'.$i.'>'; 
	}

	/**
	 * Restaure les tags HTML initialement protégés
	 *
	 * @param array $matches tableau des tags remplacés par preg_replace_callback()
	 * @return string $text le texte et les tags html correctement restitués
	 * @access protected
	 */
	protected function _restoreTags($matches)
	{
		return $this->tags[$matches[1]];
	}
}

?>